; ########################################################
; ###     ATtiny13A PWM for LED strip (2 channels)     ###
; ########################################################
; # PORTB0 - Output (ch A)                               #
; # PORTB1 - Output (ch B)                               #
; # PORTB2 - Output (linear/gamma)                       #
; # PORTB3 - Input (analog, ch A, 0 deg)                 #
; # PORTB4 - Input (analog, ch B, 180 deg)               #
; # PORTB5 - Reset (pull to VCC with 10k)                #
; ########################################################
; # Fuses (0 means programmed):                          #
; # SELFPRGEN     = 1 (default)                          #
; # DWEN          = 1 (default)                          #
; # BODLEVEL1     = 0          (changed)                 #
; # BODLEVEL0     = 0          (changed)                 #
; # RSTDISBL      = 1 (default)                          #
; # SPIEN         = 0 (default)                          #
; # EESAVE        = 1 (default)                          #
; # WDTON         = 1 (default)                          #
; # CKDIV8        = 1          (changed)                 #
; # SUT1          = 0          (changed)                 #
; # SUT0          = 0 (default)                          #
; # CKSEL1        = 1 (default)                          #
; # CKSEL0        = 0 (default)                          #
; ########################################################
; # No interrupts are used                               #
; # RAM and EEPROM is unused, excluding stack            #
; ########################################################
; # Author: Taktakishvili Vladimir (nevada@live.ru)      #
; # Last modification:    Aug 15 2014                    #
; #    PCB v1.0 done at   Mar 01 2014                    #
; # Fully compatible with Jera v1.0 PCB                  #
; ########################################################

.include "tn13def.inc"
.cseg
.org 0
RESET:	ldi R21, low(RamEnd) ; Stack init
	out SPL, R21
	
	ldi R21, 0b00000011 ; DDR for PB0 PB1 must be set as soon as possible
	out DDRB, R21 ;       to prevent gate floating !
	ser R21 ;             Protection from start flash
	out OCR0B, R21

	; Peripherals config
	ldi R21, 0b10110001 ; Timer0 config: clk/1, Phase-Correct PWM
	out TCCR0A, R21
	ldi R21, 0b00000001
	out TCCR0B, R21
	ldi R21, 0b10000111 ; ADC config
	out ADCSRA, R21

	; Energy-saving commands, can be removed if not needed
	ldi R21, 0b10000000 ; Disabling Analog Comparator
	out ACSR, R21
	ldi R21, 0b00011011 ; Disabling input logics
	out DIDR0, R21
	
	; Pins config
	ldi R21, 0b00000100
	out PORTB, R21

	ldi R26, 0x00 ; constants for calculations
	ldi R27, 0x01
	clr R0
	clr R1
	clr R2
	clr R3

	; Main loop:
Loop:	mov R23, R0
	mov R24, R2
	ldi R21, 0b00100011 ; PB3 = A ch
	out ADMUX, R21
	rcall Adcg ; Getting value from ADC
	rcall Rec ; Recalculating it
	out OCR0A, R25
	mov R0, R23
	mov R2, R24
	
	mov R23, R1
	mov R24, R3
	ldi R21, 0b00100010 ; PB4 = B ch
	out ADMUX, R21
	rcall Adcg
	rcall Rec
	com R25 ; Channel B is inverted (due phase shift)
	out OCR0B, R25
	mov R1, R23
	mov R3, R24
rjmp Loop
	
; ########################################################
; Recalculate value. Input R22, Output R25
Rec:	; Majority function (R23 and R24 are stored):
	cp R22, R23  ; Compare current value with stored in R23
	breq Rec2    ; If they are equal, increase counter R24
	cp R24, R26  ; If R24 > 0 then decrease it by 1
	breq Rec1
	dec R24
	rjmp Rec3
Rec1:	mov R23, R22
Rec2:	cpi R24, 20  ; If R24 is less, then limit - increase it
	breq Rec3
	inc R24
	; Now we have desired luminosity stored in R23
Rec3:	mov R25, R23 ; Test PING.2 input for linear dimming
	sbis PINB, 2
	rjmp Rec4
	; In this case we should load value from table
	ldi ZL, low(EyeSens*2) ; Loading start point
	ldi ZH, high(EyeSens*2)
	add ZL, R23 ; Adding shift value
	adc ZH, R26
	lpm R25, Z ; Load value from table
Rec4:
ret
; ########################################################
; Get ADC value (ADMUX already set right)
Adcg:	rcall Adcs ; First call - results ignored
	rcall Delay ; Wait for input stabilisation
	clr R4 ; Clear registers before averaging
	clr R5
	rcall Adcs ; Get value
	in R22, ADCH
	add R5, R22
;	rcall Delay
	rcall Adcs ; Get value
	in R22, ADCH
	add R5, R22
	adc R4, R26
;	rcall Delay
	rcall Adcs ; Get value
	in R22, ADCH
	add R5, R22
	adc R4, R26
;	rcall Delay
	rcall Adcs ; Get value
	in R22, ADCH
	add R5, R22
	adc R4, R26
	lsr R5
	lsr R5
	bst R4, 0
	bld R5, 6
	bst R4, 1
	bld R5, 7
;	ror R4
	mov R22, R5
ret
; ########################################################
; Starts ADC conversion and waits until it ends
Adcs:	sbi ADCSRA, ADSC ; Start conversion
Adcs1:	in R22, ADCSRA
	sbrs R22, ADIF
	rjmp Adcs1
	sbi ADCSRA, ADIF
ret
; ########################################################
; Delay for 24,641 tacts (~2.57 ms @ 9.6 MHz)
Delay:	ldi R20, 0
	ldi R19, 32
Delay1:	dec R20
	brne Delay1
	dec R19
	brne Delay1
ret

; ########################################################
; ### Data section                                     ###
; ########################################################
; This table contains eye sensetivity graph (gamma = 2.2)
EyeSens:
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01
.db 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03, 0x03, 0x04, 0x04, 0x04, 0x04, 0x05
.db 0x05, 0x05, 0x06, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07, 0x08, 0x08, 0x09, 0x09, 0x09, 0x0A, 0x0A
.db 0x0A, 0x0B, 0x0B, 0x0C, 0x0C, 0x0D, 0x0D, 0x0E, 0x0E, 0x0F, 0x0F, 0x10, 0x10, 0x11, 0x11, 0x12
.db 0x12, 0x13, 0x13, 0x14, 0x15, 0x15, 0x16, 0x16, 0x17, 0x18, 0x18, 0x19, 0x1A, 0x1A, 0x1B, 0x1C
.db 0x1C, 0x1D, 0x1E, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x22, 0x23, 0x24, 0x25, 0x26, 0x26, 0x27, 0x28
.db 0x29, 0x2A, 0x2B, 0x2C, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37
.db 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x49
.db 0x4A, 0x4B, 0x4C, 0x4D, 0x4F, 0x50, 0x51, 0x52, 0x54, 0x55, 0x56, 0x58, 0x59, 0x5A, 0x5C, 0x5D
.db 0x5E, 0x60, 0x61, 0x62, 0x64, 0x65, 0x67, 0x68, 0x69, 0x6B, 0x6C, 0x6E, 0x6F, 0x71, 0x72, 0x74
.db 0x75, 0x77, 0x79, 0x7A, 0x7C, 0x7D, 0x7F, 0x80, 0x82, 0x84, 0x85, 0x87, 0x89, 0x8A, 0x8C, 0x8E
.db 0x8F, 0x91, 0x93, 0x95, 0x96, 0x98, 0x9A, 0x9C, 0x9E, 0x9F, 0xA1, 0xA3, 0xA5, 0xA7, 0xA9, 0xAA
.db 0xAC, 0xAE, 0xB0, 0xB2, 0xB4, 0xB6, 0xB8, 0xBA, 0xBC, 0xBE, 0xC0, 0xC2, 0xC4, 0xC6, 0xC8, 0xCA
.db 0xCC, 0xCE, 0xD0, 0xD2, 0xD5, 0xD7, 0xD9, 0xDB, 0xDD, 0xDF, 0xE1, 0xE4, 0xE6, 0xE8, 0xEA, 0xED
.db 0xEF, 0xF1, 0xF3, 0xF6, 0xF8, 0xFA, 0xFD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
